(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.loulou = {}));
}(this, (function (exports) { 'use strict';

  /** Returns prototype of obj that contains the property. */
  function getPropertyOwner(obj, property) {
      if (Object.prototype.hasOwnProperty.call(obj, property)) {
          return obj;
      }
      var prototype = Object.getPrototypeOf(obj);
      return prototype ? getPropertyOwner(prototype, property) : null;
  }
  /** Inserts new prototype into the prototype chain of obj. */
  function insertPrototype(obj) {
      var proto = Object.create(Object.getPrototypeOf(obj));
      Object.setPrototypeOf(obj, proto);
      return proto;
  }
  /** Replace plain variable with getter and setter. */
  function getterize(obj, member) {
      var proto = insertPrototype(obj);
      var name = Symbol('impl');
      proto[name] = obj[member];
      delete obj[member];
      Object.defineProperty(proto, member, {
          get: function () {
              return proto[name];
          },
          set: function (val) {
              proto[name] = val;
          },
          enumerable: false,
          configurable: true
      });
  }
  function wrap(receiver, original, callback) {
      function replacement() {
          var result = original.apply(receiver, arguments);
          callback();
          return result;
      }
      Object.defineProperty(replacement, 'name', {
          value: original.name,
          writable: false
      });
      return replacement;
  }
  function hook(obj, member, callback) {
      if (obj == null) {
          return;
      }
      var owner = getPropertyOwner(obj, member);
      if (!owner)
          return;
      var descriptor = Object.getOwnPropertyDescriptor(owner, member);
      if (typeof descriptor.value !== 'function' && descriptor.get == null) {
          getterize(owner, member);
          return hook(obj, member, callback);
      }
      var original = descriptor.set || descriptor.value;
      var replacement = wrap(obj, original, callback);
      if (owner === obj) {
          obj[member] = replacement;
      }
      else {
          var proto = insertPrototype(obj);
          if (descriptor.set) {
              Object.defineProperty(proto, member, {
                  get: descriptor.get,
                  set: replacement
              });
          }
          else {
              proto[member] = replacement;
          }
      }
  }

  function toElem(html) {
      var template = document.createElement('template');
      template.innerHTML = html;
      var fragment = template.content;
      if (fragment.childElementCount !== 1) {
          throw new Error('html should contain exactly one top-level element');
      }
      return fragment.firstElementChild;
  }
  function elemTo$(element) {
      return function (selector) {
          return !selector ? element : element.querySelector(selector);
      };
  }
  function to$(html) {
      return elemTo$(toElem(html));
  }

  var Ui = /** @class */ (function () {
      function Ui(updater) {
          this.updater = updater;
          this.$ = null;
      }
      Ui.prototype.update = function (callback) {
          var self = this;
          return function () {
              callback.apply(undefined, arguments);
              self.updater(self.$);
          };
      };
      Ui.prototype.watch = function (receiver, method) {
          var _this = this;
          hook(receiver, method, function () { return _this.updater(_this.$); });
      };
      return Ui;
  }());
  function render(component, container) {
      if (component instanceof Element) {
          return container.appendChild(component);
      }
      else {
          var ui = component.ui;
          var element = component.render();
          ui.$ = elemTo$(element);
          return container.appendChild(element);
      }
  }

  exports.Ui = Ui;
  exports.elemTo$ = elemTo$;
  exports.hook = hook;
  exports.render = render;
  exports.to$ = to$;
  exports.toElem = toElem;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
